
#include "consoleui.hh"

#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <string.h>

#include <game/position.hh>
#include <tictactoe/tttstate.hh>
#include <fourinrow/fourstate.hh>

#include <errno.h>
extern int errno;


ConsoleUI::ConsoleUI() {

}

ConsoleUI::~ConsoleUI() {

}

int ConsoleUI::initialize() {
  return 0;
}

int ConsoleUI::run() {
  doMenu();
  return 0;
}

void ConsoleUI::doMenu() {
  char q;

  do {
    printf("\
** Main menu **\n\
t ... TicTacToe\n\
4 ... Four in a Row\n\
p ... Define players\n\
q ... Quit\n\
Do what? ");
    
    q=(char)getchar(); /* clean the CR */ getchar();
    switch (q) {
    case 'q':
      break;
    case 't':
      {
	if (getPlayer(0)==NULL || getPlayer(1)==NULL)
	  printf("Define players first!\n");
	else {
	  setGameState(new TTTState());
	  doTTTMenu();
	}
      }
      break;
    case '4':
    {
	    if (getPlayer(0)==NULL || getPlayer(1)==NULL)
		    printf("Define players first!\n");
	    else {
		    setGameState(new FourState());
		    doFourMenu();
	    }
    }
    break;
    case 'p':
    {
	    doPlayersMenu();
    }
    break;
    default:
	    break;
    }    
  } while (q!='q');
}

void ConsoleUI::doTTTMenu() {
  char q=1;
  int currplayer=0;
  
  do {
    printf("** TicTacToe **\n");

    printField(getGameState()->getField());
    
    if (q!='p')
      printf("\
p ... Play\n\
s ... Suggest\n\
q ... Quit\n\
Do what? ");
    
    q=(char)getchar(); /* clean the CR */ getchar();
    switch (q) {
    case 'q':
      break;
    case 'p':
      {
	Player *p=getPlayer(currplayer);
	GameState *gs=getGameState();
	Position pos;

	if (p->getAI()) {
	  pos=p->getMove(gs);
	} else {
		printf("%s, do your move!\n", p->getName());
		pos=getMove(p);
		printf("You chose: [%i, %i]\n", pos.getH(), pos.getW());
	}
	
	if (gs->performMove(p->getID(), pos)) {
		currplayer=!currplayer;
		long eval=gs->evaluate(0, 1);
		gs->isFinal(0, 1);
		printf("Evaluation: %i\n", eval);
		if (gs->getWinner()>-1) {
			if (gs->getWinner()<2)
				printf("Game over! Winner is %s!\n", getPlayer(gs->getWinner())->getName());
			else
				printf("Game over! Game is a DRAW!\n");
			q='q';
		}
	}
	else
		printf("Move is not allowed!\n");
      }      
      break;
      
    case 's':
      {
	int brainsize;
	Player *p=getPlayer(currplayer);
	GameState *gs=getGameState();
	printf("Brainsize? ");
	cin >> brainsize;  /* clean the CR */ getchar();
	p->setBrainsize(brainsize);
	Position pos=p->getMove(gs);
	printf("I suggest@%i: [%i, %i]\n", p->getBrainsize(), pos.getH(), pos.getW());
      }
      break;

    default:
      break;
    }
  } while (q!='q');
  printField(getGameState()->getField());
}


void ConsoleUI::doFourMenu() {
  char q=1;
  int currplayer=0;
  
  do {
    printf("** Four-in-a-Row **\n");

    printField(getGameState()->getField());
    
    if (q!='p')
      printf("\
p ... Play\n\
s ... Suggest\n\
q ... Quit\n\
Do what? ");
    
    q=(char)getchar(); /* clean the CR */ getchar();
    switch (q) {
    case 'q':
      break;
    case 'p':
      {
	Player *p=getPlayer(currplayer);
	GameState *gs=getGameState();
	Position pos;

	if (p->getAI()) {
	  pos=p->getMove(gs);
	} else {
	  printf("%s, do your move!\n", p->getName());
	  pos=getMoveF();
	  printf("You chose: [%i]\n", pos.getW());
	}
	
	if (gs->performMove(p->getID(), pos)) {
	  currplayer=!currplayer;
	  long eval=gs->evaluate(0, 1);
	  gs->isFinal(0,1);
	  printf("Evaluation: %i Winner: %i\n", eval, gs->getWinner());
	  if (gs->getWinner()>-1) {
	    if (gs->getWinner()<2)
	      printf("Game over! Winner is %s!\n", getPlayer(gs->getWinner())->getName());
	    else
	      printf("Game over! Game is a DRAW!\n");
	    q='q';
	  }
	}
	else
	  printf("Move is not allowed!\n");
      }      
      break;

    case 'm':
      {
	int poscount;
	Position *pos=getGameState()->getAllowedMoves(poscount);
	printf("%i available col: ", poscount);
	for (int i=0; i<poscount; i++) {
	  printf("%i ", pos[i].getW());
	}
	printf("\n");
      }
      break;

    case 's':
      {
	int brainsize;
	Player *p=getPlayer(currplayer);
	GameState *gs=getGameState();
	printf("Brainsize? ");
	cin >> brainsize;  /* clean the CR */ getchar();
	p->setBrainsize(brainsize);
	Position pos=p->getMove(gs);
	printf("I suggest@%i: [%i]\n", p->getBrainsize(), pos.getW());
      }
      break;

    default:
      break;
    }
  } while (q!='q');
  printField(getGameState()->getField());
}


void ConsoleUI::doPlayersMenu() {
    char q;
  
  do {
    printf("** Players **\n");
    printPlayers();
    printf("\
1 ... Player 1\n\
2 ... Player 2\n\
q ... Quit\n\
Do what? ");

    q=(char)getchar(); /* clean the CR */ getchar();

    switch (q) {
    case 'q':
      break;
    case '1':
      {
	Player *p=new Player();
	char buff[80];
	printf("Is AI? ");
	cin >> buff[0]; /* clean the CR */ getchar();
	if (buff[0]=='y') {
	  p->setAI(true);
	  int brainsize;
	  printf("Brainsize? ");
	  cin >> brainsize;  /* clean the CR */ getchar();
	  p->setBrainsize(brainsize);
	}
	printf("Name: ");
	cin >> buff; /* clean the CR */ getchar();
	p->setName(strdup(buff));
	setPlayer(0, p);
      }
      break;

    case '2':
      {
	Player *p=new Player();
	char buff[80];
	printf("Is AI? ");
	cin >> buff[0]; /* clean the CR */ getchar();
	if (buff[0]=='y') {
	  p->setAI(true);
	  int brainsize;
	  printf("Brainsize? ");
	  cin >> brainsize;  /* clean the CR */ getchar();
	  p->setBrainsize(brainsize);
	}
	printf("Name: ");
	cin >> buff; /* clean the CR */ getchar();
	p->setName(strdup(buff));
	setPlayer(1, p);
      }
      break;
      
    default:
      break;
    }
  } while (q!='q');
    
}
  
Position ConsoleUI::getMove(Player*) {
  Position pos;
  printf("Row: ");
  pos.setH(readval(2));
  printf("Column: ");
  pos.setW(readval(2));
  return pos;
}

Position ConsoleUI::getMoveF() {
  Position pos;
  printf("Column: ");
  pos.setW(readval(6));
  return pos;
}

int ConsoleUI::readval(int max) {
  int val=-1;
  
  int erange=-1, evalue=-2;
  
  char buff[5], *fin;
    
  do {
    
    try {

      cin >> buff; /* clean the CR */ getchar();

      val=strtol((char*)buff, &fin, 10);
      if (val<0 || val>max || errno==ERANGE)
	throw erange;
      if (*fin!='\0' && *fin!='\n')
	throw evalue;

    } catch (int evalue) {
      printf("Invalid number! Retry: ");
      val=-1;
    }
  } while (val==-1);
  
  return val;
}


void ConsoleUI::printField(GameField* field) {
  for (int i=0; i<field->getHeight(); i++) {
    for (int j=0; j<field->getWidth(); j++) {
      int val=field->getValue(j, i);
      char c=' ';
      switch (val) {
      case 0:
	c='X';
	break;
      case 1:
	c='O';
	break;
      default:
	c=' ';
      }	
      printf("|%c", c);
    }
    printf("|\n");
  }
}

void ConsoleUI::printPlayers() {
  Player *p=getPlayer(0);
  if (p==NULL)
    printf("Player 1 is not defined.\n");
  else 
    printf("Player 1 is '%s' identified by '%i'.\n", p->getName(), p->getID());

  p=getPlayer(1);
  if (p==NULL)
    printf("Player 2 is not defined.\n");
  else 
    printf("Player 2 is '%s' identified by '%i'.\n", p->getName(), p->getID());

}
