 
#include "fourstate.hh"
#include "fourfield.hh"

#include <stdio.h>

#define value_at(row, col)     	data[(col)*fieldheight+(row)]

FourState::FourState() {
  m_field=new FourField();
}

FourState::~FourState() {
  delete m_field;
}

GameState* FourState::duplicate() {
  FourState* nstate=new FourState();
  memcpy(nstate->m_field->getData(), m_field->getData(), sizeof(char)*7*12);
  return nstate;
}

bool FourState::performMove(int id, Position &pos) {
  int fieldheight=m_field->getHeight();

  for (int i=fieldheight-1; i>=0; i--) 
    //for (int i=0; i<fieldheight; i++)
    if (m_field->getValue(pos.getW(), i)==-1) {
      m_field->setValue(id, pos.getW(), i);
      pos.setH(i);
      return true;
    }
  
  return false;
}

bool FourState::isFinal(int player, int enemy) {

  int mycount, enemycount;
  int row, col;

  char* data=m_field->getData();
  int fieldwidth=m_field->getWidth(),
    fieldheight=m_field->getHeight();

  
  // Count  R-O-W  possibilities
  for (row=0; row<fieldheight; row++)
    for (col=0; col+3<fieldwidth; col++)
      {
	mycount=count(player, value_at(row, col), value_at(row, col+1), value_at(row, col+2), value_at(row, col+3));
	enemycount=count(enemy, value_at(row, col), value_at(row, col+1), value_at(row, col+2), value_at(row, col+3));
	
	if (mycount==4) {
	  setWinner(player);
	  return true;
	}
	
	if (enemycount==4) {
	  setWinner(enemy);
	  return true;
	}
      }
  
  // count  C-O-L-U-M-N  possibilities
  for (col=0; col<fieldwidth; col++)
    for (row=0; row+3<fieldheight; row++)
      {
	mycount=count(player, value_at(row, col), value_at(row+1, col), value_at(row+2, col), value_at(row+3, col));
	enemycount=count(enemy, value_at(row, col), value_at(row+1, col), value_at(row+2, col), value_at(row+3, col));
	
	if (mycount==4) {
	  setWinner(player);
	  return true;
	}

	if (enemycount==4) {
	  setWinner(enemy);
	  return true;
	}
      }
  
  // Check  D-I-A-G-O-N-A-L  possibilities    -->   '/'
  
  for (row=3; row<fieldheight; row++)
	for (col=3; col<fieldwidth; col++)
	  {
	    mycount=count(player, value_at(row, col), value_at(row-1, col-1), value_at(row-2, col-2), value_at(row-3, col-3));
	    enemycount=count(enemy, value_at(row, col), value_at(row-1, col-1), value_at(row-2, col-2), value_at(row-3, col-3));
	    
	    if (mycount==4) {
	      setWinner(player);
	      return true;
	    }
	    if (enemycount==4) {
		setWinner(enemy);
		return true;
	    }
	  }
  
  // Check other  D-I-A-G-O-N-A-L  possibilities  -->  '\'
  
  for (row=3; row<fieldheight; row++)
    for (col=0; col+3<fieldwidth; col++)
      {
	mycount=count(player, value_at(row, col), value_at(row-1, col+1), value_at(row-2, col+2), value_at(row-3, col+3));
	enemycount=count(enemy, value_at(row, col), value_at(row-1, col+1), value_at(row-2, col+2), value_at(row-3, col+3));
	
	if (mycount==4) {
	  setWinner(player);
	  return true;
	}
	
	if (enemycount==4) {
	  setWinner(enemy);
	  return true;
	}
      }
  
  
  return false;
}

long FourState::evaluate(int player, int enemy) {

  short heu=0;
  int mycount, enemycount;
  int row, col;

  char* data=m_field->getData();
  int fieldwidth=m_field->getWidth(),
    fieldheight=m_field->getHeight();

  
  // Count  R-O-W  possibilities
  for (row=0; row<fieldheight; row++)
    for (col=0; col+3<fieldwidth; col++)
      {
	mycount=count(player, value_at(row, col), value_at(row, col+1), value_at(row, col+2), value_at(row, col+3));
	enemycount=count(enemy, value_at(row, col), value_at(row, col+1), value_at(row, col+2), value_at(row, col+3));
	
	if (mycount==1 && enemycount==0)
	  heu+=1;
	if (mycount==0 && enemycount==1)
	  heu-=1;
	
	if (mycount==2 && enemycount==0)
	  heu+=10;
	if (mycount==0 && enemycount==2)
	  heu-=10;
	
	if (mycount==3 && enemycount==0)
	  heu+=50;
	if (mycount==0 && enemycount==3)
	  heu-=50;
	
	if (mycount==4)  {
	  setWinner(player);
	  heu+=2000;
	}
	if (enemycount==4) {
	  setWinner(enemy);
	  heu-=2000;
	}
      }
  
  // count  C-O-L-U-M-N  possibilities
  for (col=0; col<fieldwidth; col++)
    for (row=0; row+3<fieldheight; row++)
      {
	mycount=count(player, value_at(row, col), value_at(row+1, col), value_at(row+2, col), value_at(row+3, col));
	enemycount=count(enemy, value_at(row, col), value_at(row+1, col), value_at(row+2, col), value_at(row+3, col));
	
	if (mycount==1 && enemycount==0)
	  heu+=1;
	if (mycount==0 && enemycount==1)
	  heu-=1;
	
	if (mycount==2 && enemycount==0)
	  heu+=10;
	if (mycount==0 && enemycount==2)
	  heu-=10;
	
	if (mycount==3 && enemycount==0)
	  heu+=50;
	if (mycount==0 && enemycount==3)
	  heu-=50;
	
	if (mycount==4) {
	  setWinner(player);
	  heu+=2000;
	}
	if (enemycount==4) {
	  setWinner(enemy);
	  heu-=2000;
	}
      }
  
  // Check  D-I-A-G-O-N-A-L  possibilities    -->   '/'
  
  for (row=3; row<fieldheight; row++)
	for (col=3; col<fieldwidth; col++)
	  {
	    mycount=count(player, value_at(row, col), value_at(row-1, col-1), value_at(row-2, col-2), value_at(row-3, col-3));
	    enemycount=count(enemy, value_at(row, col), value_at(row-1, col-1), value_at(row-2, col-2), value_at(row-3, col-3));
	    
	    if (mycount==1 && enemycount==0)
	      heu+=1;
	    if (mycount==0 && enemycount==1)
	      heu-=1;
	    
	    if (mycount==2 && enemycount==0)
	      heu+=10;
	    if (mycount==0 && enemycount==2)
	      heu-=10;
	    
	    if (mycount==3 && enemycount==0)
	      heu+=50;
	    if (mycount==0 && enemycount==3)
	      heu-=50;
	    
	    if (mycount==4) {
	      setWinner(player);
	      heu+=2000;
	    }
	    if (enemycount==4) {
	      setWinner(enemy);
	      heu-=2000;
	    }
	  }
  
  // Check other  D-I-A-G-O-N-A-L  possibilities  -->  '\'
  
  for (row=3; row<fieldheight; row++)
	for (col=0; col+3<fieldwidth; col++)
	  {
	    mycount=count(player, value_at(row, col), value_at(row-1, col+1), value_at(row-2, col+2), value_at(row-3, col+3));
	    enemycount=count(enemy, value_at(row, col), value_at(row-1, col+1), value_at(row-2, col+2), value_at(row-3, col+3));
	    
	    if (mycount==1 && enemycount==0)
	      heu+=1;
	    if (mycount==0 && enemycount==1)
	      heu-=1;
	    
	    if (mycount==2 && enemycount==0)
	      heu+=10;
	    if (mycount==0 && enemycount==2)
	      heu-=10;
	    
	    if (mycount==3 && enemycount==0)
	      heu+=50;
	    if (mycount==0 && enemycount==3)
	      heu-=50;
	    
	    if (mycount==4) {
	      setWinner(player);
	      heu+=2000;
	    }
	    if (enemycount==4) {
	      setWinner(enemy);
	      heu-=1000;
	    }
	  }
 
  
  setEval(heu);
  return heu;
}
/*
  void FourState::setWinner(int winner) {
  m_winner=winner;
  }
*/
int FourState::count(int pattern, int a, int b, int c, int d)
{
  int ret=0;
  if (a==pattern) ret++;
  if (b==pattern) ret++;
  if (c==pattern) ret++;
  if (d==pattern) ret++;
  
  return ret;
}

Position* FourState::getAllowedMoves(int &count) {
  count=getAllowedCount();
  int tcount=0;
  if (count==0)
    return NULL;

  Position *pos=new Position[count];
  char *data=m_field->getData();
  int fieldwidth=m_field->getWidth(), 
    fieldheight=m_field->getHeight();

  // cols
  for (int i=0; i<fieldwidth; i++) {
    // rows
    if (value_at(0, i)==-1) {
      pos[tcount++].setW(i);
    }
  }
  
  return pos;
}

int FourState::getAllowedCount() {
  if (getWinner()!=-1)
    return 0;
  int maxcount=0, fieldheight=m_field->getHeight();

  char *data=m_field->getData();

  for (int col=0; col<7; col++)
    if (value_at(0, col)==-1)
      maxcount++;

  if (maxcount==0)
    setWinner(2);
  
  return maxcount;
}

GameField* FourState::getField() {
  return m_field;
}


