
#include <gtk/gtk.h>
#include "gtkfieldview.hh"
#include <stdlib.h>
#include <stdio.h>

GtkFieldView::GtkFieldView()
  : Gtk::DrawingArea(), pixmap(0)
{
  set_events (GDK_EXPOSURE_MASK
	      | GDK_LEAVE_NOTIFY_MASK
	      | GDK_BUTTON_PRESS_MASK
	      | GDK_BUTTON_RELEASE_MASK
	      | GDK_POINTER_MOTION_MASK
	      | GDK_POINTER_MOTION_HINT_MASK);

  field=NULL;
}

GtkFieldView::~GtkFieldView() {
}

void GtkFieldView::setGameField(GameField *fld)
{
  gameend=false;
  field=fld;
  calculateCell(width(), height());
  draw(NULL);
}

void GtkFieldView::gameEnd() {
  gameend=true;
  draw(NULL);
}

void GtkFieldView::gameStateChange(int row, int col) {
  //  int x=(width()-cellw*field->getWidth())/2, y=(height()-cellw*field->getHeight())/2;  
  //  QRect box(col*cellw, row*cellw, cellw, cellw);
  // box.moveBy(x, y);
  //  repaint(box, TRUE);
  paintField();
  draw(NULL);
}


/* Create a new backing pixmap of the appropriate size */
int GtkFieldView::configure_event_impl(GdkEventConfigure *event) {
  win = get_window();
  visual = win.get_visual();
  
  if (pixmap)
    pixmap.release();
  
  pixmap.create(get_window(), width(), height());

  calculateCell(width(), height());
  clear();
  draw(NULL);
  return TRUE;
}

void GtkFieldView::clear() {
  gc = get_style()->gtkobj()->white_gc;

  pixmap.draw_rectangle (gc,
			 TRUE,
			 0, 0,
			 width(),
			 height());
  
}

/* Redraw the screen from the backing pixmap */
int GtkFieldView::expose_event_impl (GdkEventExpose *event)
{
  gc = get_style()->gtkobj()->fg_gc[GTK_WIDGET_STATE (GTK_WIDGET(gtkobj()))];
  // Same like above, + Gtk::Widget has set_state function but no get_state 
  // function.
  if (field!=NULL) {
    paintField();
    win.draw_pixmap(gc,
		    pixmap,
		    event->area.x, event->area.y,
		    event->area.x, event->area.y,
		    event->area.width, event->area.height);
  } else {
    
  }
  
  return FALSE;
}

gint GtkFieldView::button_press_event_impl (GdkEventButton *event) {
  if (gameend)
    gameend=false;
  return TRUE;
}

gint GtkFieldView::button_release_event_impl (GdkEventButton *ev)
{
  if (field!=NULL) {
    int x=((int)ev->x)-(width()-cellw*field->getWidth())/2, y=((int)ev->y)-(height()-cellw*field->getHeight())/2;  
    positionClicked.emit(x/cellw, y/cellw);
  }
  return TRUE;
}

void GtkFieldView::paintField() {
  int x=(width()-cellw*field->getWidth())/2, y=(height()-cellw*field->getHeight())/2;  
  Gdk_GC gc=Gdk_GC(pixmap);

  clear();
  Gdk_Color blue("blue"), red("red"), black("black");
  get_colormap().alloc(red);
  get_colormap().alloc(blue);
  get_colormap().alloc(black);
  
  // rows
  for (int i=0; i<field->getHeight(); i++) {
    // cols
    for (int j=0; j<field->getWidth(); j++) {
      GdkRectangle bounds;
      GdkRectangle box;

      bounds.x=x+j*cellw+2; bounds.y=y+i*cellw+2; bounds.width=cellw-4; bounds.height=cellw-4;
      box.x=x+j*cellw; box.y=y+i*cellw; box.width=cellw; box.height=cellw;

      int val=field->getValue(j, i);
      switch (val) {
      case 0: //  X
	gc.set_foreground(red);
	gdk_draw_arc(pixmap, gc, TRUE, bounds.x, bounds.y, bounds.width, bounds.height, 0, 23040);
	break;
      case 1: //  O
	gc.set_foreground(blue);
	gdk_draw_arc(pixmap, gc, TRUE, bounds.x, bounds.y, bounds.width, bounds.height, 0, 23040);
	break;
      default:
	break;
      }	
      gc.set_foreground(black);
      gdk_draw_rectangle(pixmap, gc, FALSE, box.x, box.y, box.width, box.height);
    }
  }
}


void GtkFieldView::calculateCell(int width, int height) {
  if (field!=NULL) {
    int nw=width-20, nh=height-20;
    
    if (field!=NULL) {
      cellw=nw/field->getWidth();
      nw=cellw*field->getWidth();
      nh=cellw*field->getHeight();
      if (nh>height) {
	cellw=(height-20)/field->getHeight();
      }
    }
    if (cellw<1) cellw=1;
  }
}
