
#include "player.hh"

#include <stdlib.h>
#include <string.h>

Player::Player(int id) {
  // inicializacija
  m_name=NULL;
  m_score=0;
  m_id=id;
  m_amai=false;
  m_brainsize=1;
}

Player::~Player() {
  // sprosti pomnilnik
  if (m_name!=NULL)
    delete m_name;
}

void Player::setID(int id) {
  m_id=id;
}

int Player::getID() {
  return m_id;
}

void Player::setAI(bool isai) {
  m_amai=isai;
}

bool Player::getAI() {
  return m_amai;
}

void Player::setBrainsize(int bs) {
  m_brainsize=bs;
}

int Player::getBrainsize() {
  return m_brainsize;
}

void Player::setName(char* name) {
  // sprosti pomnilnik
  if (m_name!=NULL)
    delete m_name;
  
  // kopija char*
  m_name=strdup(name);
}

char* Player::getName() {
  // vrni kazalec
  return m_name;
}

void Player::incrementScore(long by) {
  m_score+=by;
}

long Player::getScore() {
  return m_score;
}

void Player::resetScore() {
  m_score=0;
}

Position Player::getMove(GameState *state) {
  //return getMoveMinimax(state);
  long beta=100000, alpha=-beta;
  return getMoveAlphaBeta(state, alpha, beta);
}

Position Player::getMoveMinimax(GameState *state, int depth) {
  
  Position bestpos;
  // node is a leaf
  if (depth==m_brainsize || state->isFinal(m_id, !m_id)) {
    state->evaluate(m_id, !m_id);
    return bestpos;
  }
  
  long minimum=1000000, maximum=-minimum;
  
  int poscount;
  Position *pos=state->getAllowedMoves(poscount);
  
  // node is a leaf
  if (poscount==0) {
    state->evaluate(m_id, !m_id);
    return bestpos; 
  }
  
  for (int i=0; i<poscount; i++) {
    GameState* nstate=state->duplicate();
    nstate->performMove((depth%2==0?m_id:!m_id), pos[i]);
    getMoveMinimax(nstate, depth+1);
    if ((depth%2)==0) { // MAX NODE
      if (nstate->getEval()>maximum || nstate->getWinner()==m_id) {
	maximum=nstate->getEval();
	bestpos=pos[i];
      }
    } else { // MIN NODE
      if (nstate->getEval()<minimum || nstate->getWinner()==m_id) {
	minimum=nstate->getEval();
	bestpos=pos[i];
      }
    }
    delete nstate;
  }
  
   
  delete []pos;
  state->setEval((depth%2==0?maximum:minimum));
  return bestpos;
}

Position Player::getMoveAlphaBeta(GameState *state, long alpha, long beta, int depth) {
  
  Position bestpos;
  if (depth==m_brainsize || state->isFinal(m_id, !m_id)) {
    state->evaluate(m_id, !m_id);
    return bestpos;
  }

  //  int minimum=10000, maximum=-10000;
  
  int poscount;
  Position *pos=state->getAllowedMoves(poscount);
  
  if (poscount==0) {
    state->evaluate(m_id, !m_id);
    return bestpos;    
  }
  
  GameState* nstate=NULL;
  
  if ((depth%2)==1) { // MIN NODE
    /*
      For each successor Ni of N loop
	Let Val be MINIMAX-AB(Ni, Alpha of N, Beta of N);
	Set Beta value of N to Min{Beta value of N, Val};
	When Beta value of N <= Alpha value of N then exit loop;
      Return Beta value of N;
    */
    for (int i=0; i<poscount && beta > alpha; i++) {
      
      nstate=state->duplicate();
      nstate->performMove((depth%2==0?m_id:!m_id), pos[i]);
      getMoveAlphaBeta(nstate, alpha, beta, depth+1);
      if (nstate->getEval()<beta) {
	beta=nstate->getEval();
	bestpos=pos[i];
	if (beta<=alpha || nstate->getWinner()==!m_id) {
	  delete nstate;
	  delete []pos;
	  state->setEval((depth%2==1?beta:alpha));
	  return bestpos;
	}
      }
      delete nstate;
    }
    
  } else { // MAX NODE
    /*
      For each successor Ni of N loop
        Let Val be MINIMAX-AB(Ni, Alpha of N, Beta of N);
	Set Alpha value of N to Max{Alpha value of N, Val};
	When Alpha value of N >= Beta value of N then exit loop;
      Return Alpha value of N;
    */
    for (int i=0; i<poscount && alpha < beta; i++) {
      
      nstate=state->duplicate();
      nstate->performMove((depth%2==0?m_id:!m_id), pos[i]);
      getMoveAlphaBeta(nstate, alpha, beta, depth+1);
      
      if (nstate->getEval()>alpha) {
	alpha=nstate->getEval();
	bestpos=pos[i];
	if (alpha>=beta || nstate->getWinner()==m_id) {
	  delete nstate;
	  delete []pos;
	  state->setEval((depth%2==1?beta:alpha));
	  return bestpos;
	}
      }
      delete nstate;
    }
  }
  
  delete []pos;
  state->setEval((depth%2==1?beta:alpha));
  return bestpos;
}
