
#include "fieldview.hh"
#include <qcolor.h>
#include <qbrush.h>
#include <qpainter.h>
#include <iostream.h>
#include <stdlib.h>


QFieldView::QFieldView(QWidget *parent, const char *name)
  : QWidget(parent, name)
{
  field=NULL;
  setPalette(QPalette(QColor(250, 250, 250)));
}

void QFieldView::setGameField(GameField *fld)
{
  gameend=false;
  field=fld;
  calculateCell(width(), height());
  repaint();
}

void QFieldView::gameEnd() {
  gameend=true;
  // repaint();
}

void QFieldView::gameStateChange(int row, int col) {
  int x=(width()-cellw*field->getWidth())/2, y=(height()-cellw*field->getHeight())/2;  
  QRect box(col*cellw, row*cellw, cellw, cellw);
  box.moveBy(x, y);
  repaint(box, TRUE);
}

void QFieldView::mousePressEvent (QMouseEvent *) {
  if (gameend)
    gameend=false;
}

void QFieldView::mouseReleaseEvent (QMouseEvent *ev) {
  if (field!=NULL) {
    int x=ev->x()-(width()-cellw*field->getWidth())/2, y=ev->y()-(height()-cellw*field->getHeight())/2;  
    emit positionClicked(x/cellw, y/cellw);
  }
}

void QFieldView::paintField(QPaintEvent *ev) {
  int x=(width()-cellw*field->getWidth())/2, y=(height()-cellw*field->getHeight())/2;  

  pixmap.resize(width(), height());

  pixmap.fill(backgroundColor());
  QPainter p(&pixmap);
  p.translate(x, y);
  p.setPen(black);  
  // rows
  for (int i=0; i<field->getHeight(); i++) {
    // cols
    for (int j=0; j<field->getWidth(); j++) {
      QRect bounds(j*cellw+2, i*cellw+2, cellw-4, cellw-4);
      QRect box(j*cellw, i*cellw, cellw+1, cellw+1);
      int val=field->getValue(j, i);
      switch (val) {
      case 0: //  X
	p.setBrush(red);
	p.drawEllipse(bounds);
	break;
      case 1: //  O
	p.setBrush(blue);
	p.drawEllipse(bounds);
	break;
      default:
	break;
      }	
      p.setBrush(NoBrush);
      p.drawRect(box);
    }
  }
  
  QPainter p2(this);
  p2.setClipRect(ev->rect());
  p2.drawPixmap(0,0,pixmap);
}

void QFieldView::paintEvent(QPaintEvent *ev)
{
  if (field!=NULL) {
    paintField(ev);
  } else {
    QPainter p(this);
    p.drawText(20, 20, ":-)");
  }
}

QSizePolicy QFieldView::sizePolicy() const
{
  return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}

void QFieldView::resizeEvent(QResizeEvent *ev) {
  calculateCell(ev->size().width(), ev->size().height());
}

void QFieldView::calculateCell(int width, int height) {
  if (field!=NULL) {
    int nw=width-20, nh=height-20;
    
    if (field!=NULL) {
      cellw=nw/field->getWidth();
      nw=cellw*field->getWidth();
      nh=cellw*field->getHeight();
      if (nh>height) {
	cellw=(height-20)/field->getHeight();
      }
    }
    if (cellw<1) cellw=1;
  }
}
