
#include "tttstate.hh"
#include "tttfield.hh"

#include <stdio.h>

TTTState::TTTState() {
  m_field=new TTTField();
}

TTTState::~TTTState() {
  delete m_field;
}

GameState* TTTState::duplicate() {
  TTTState* nstate=new TTTState();
  memcpy(nstate->m_field->getData(), m_field->getData(), sizeof(char)*9);
  return nstate;
}

bool TTTState::performMove(int id, Position &pos) {
  if (m_field->getValue(pos.getW(), pos.getH())!=-1)
    return false;

  m_field->setValue(id, pos.getW(), pos.getH());
  return true;
}

bool TTTState::isFinal(int player, int enemy) {

  int mycount, enemycount, i;
  
  char* data=m_field->getData();
  int fieldwidth=m_field->getWidth();
  
  // Count  R-O-W  possibilities
  for (i=0; i<3; i++) {
    mycount=count(player, data[i*fieldwidth], data[i*fieldwidth+1], data[i*fieldwidth+2]);
    enemycount=count(enemy, data[i*fieldwidth], data[i*fieldwidth+1], data[i*fieldwidth+2]);
    
    if (mycount==3) {
      setWinner(player);
      return true;
    }
    
    if (enemycount==3) {
      setWinner(enemy);
      return true;
    }
  }
  
  // count  C-O-L-U-M-N  possibilities
  for (i=0; i<3; i++) {
    mycount=count(player, data[i], data[fieldwidth+i], data[2*fieldwidth+i]);
    enemycount=count(enemy, data[i], data[fieldwidth+i], data[2*fieldwidth+i]);
    
    if (mycount==3) {
      setWinner(player);
      return true;
    }
    
    if (enemycount==3) {
      setWinner(enemy);
      return true;
    }
  }
  
  // Check  D-I-A-G-O-N-A-L  possibilities
  {
    mycount=count(player, data[0], data[fieldwidth+1], data[2*fieldwidth+2]);
    enemycount=count(enemy, data[0], data[fieldwidth+1], data[2*fieldwidth+2]);
    
    if (mycount==3) {
      setWinner(player);
      return true;
    }
    
    if (enemycount==3) {
	setWinner(enemy);
	return true;
    }
  }
  
  // Check other  D-I-A-G-O-N-A-L  possibilities
  {
    mycount=count(player, data[2], data[fieldwidth+1], data[2*fieldwidth]);
    enemycount=count(enemy, data[2], data[fieldwidth+1], data[2*fieldwidth]);
    
    if (mycount==3) {
      setWinner(player);
      return true;
    }
    
    if (enemycount==3) {
      setWinner(enemy);
      return true;
    }
  }
  
  return false;
}

long TTTState::evaluate(int player, int enemy) {

  short heu=0;
  int mycount, enemycount, i;

  char* data=m_field->getData();
  int fieldwidth=m_field->getWidth();

  // Count  R-O-W  possibilities
  for (i=0; i<3; i++) {
    mycount=count(player, data[i*fieldwidth], data[i*fieldwidth+1], data[i*fieldwidth+2]);
    enemycount=count(enemy, data[i*fieldwidth], data[i*fieldwidth+1], data[i*fieldwidth+2]);
    
    if (mycount==1 && enemycount==0)
      heu+=2;
    if (mycount==0 && enemycount==1)
      heu-=2;
    
    if (mycount==2 && enemycount==0)
      heu+=10;
    if (mycount==0 && enemycount==2)
      heu-=10;
    
    if (mycount==3)
      {
	setWinner(player);
	heu+=200;
	setEval(heu);
	return heu;
      }
    if (enemycount==3)
      {
	setWinner(enemy);
	heu-=200;
	setEval(heu);
	return heu;
      }
  }
  
  // count  C-O-L-U-M-N  possibilities
  for (i=0; i<3; i++) {
    mycount=count(player, data[i], data[fieldwidth+i], data[2*fieldwidth+i]);
    enemycount=count(enemy, data[i], data[fieldwidth+i], data[2*fieldwidth+i]);
    
    if (mycount==1 && enemycount==0)
      heu+=2;
    if (mycount==0 && enemycount==1)
      heu-=2;
    
    if (mycount==2 && enemycount==0)
      heu+=10;
    if (mycount==0 && enemycount==2)
      heu-=10;
    
    if (mycount==3)
      {
	setWinner(player);
	heu+=200;
        setEval(heu);
	return heu;
      }
    if (enemycount==3)
      {
	setWinner(enemy);
	heu-=200;
	setEval(heu);
	return heu;      
      }
  }
  
  // Check  D-I-A-G-O-N-A-L  possibilities
  {
    mycount=count(player, data[0], data[fieldwidth+1], data[2*fieldwidth+2]);
    enemycount=count(enemy, data[0], data[fieldwidth+1], data[2*fieldwidth+2]);
    
    if (mycount==1 && enemycount==0)
      heu+=2;
    if (mycount==0 && enemycount==1)
      heu-=2;
    
    if (mycount==2 && enemycount==0)
      heu+=10;
    if (mycount==0 && enemycount==2)
      heu-=10;
    
    if (mycount==3)
      {
	setWinner(player);
	heu+=200;
	setEval(heu);
	return heu;
      }
    if (enemycount==3)
      {
	setWinner(enemy);
	heu-=200;
	setEval(heu);
	return heu;      
      }
  }
  
  // Check other  D-I-A-G-O-N-A-L  possibilities
  {
    mycount=count(player, data[2], data[fieldwidth+1], data[2*fieldwidth]);
    enemycount=count(enemy, data[2], data[fieldwidth+1], data[2*fieldwidth]);
    
    if (mycount==1 && enemycount==0)
      heu+=2;
    if (mycount==0 && enemycount==1)
      heu-=2;
    
    if (mycount==2 && enemycount==0)
      heu+=10;
    if (mycount==0 && enemycount==2)
      heu-=10;
    
    if (mycount==3)
      {
	setWinner(player);
	heu+=200;
	setEval(heu);
	return heu;
      }
    if (enemycount==3)
      {
	setWinner(enemy);
	heu-=200;
	setEval(heu);
	return heu;
      }
  }
  
  setEval(heu);
  return heu;
}
/*
  void TTTState::setWinner(int winner) {
  m_winner=winner;
  }
*/
int TTTState::count(int pattern, int a, int b, int c)
{
  int ret=0;
 
  if (a==pattern) ret++;
  if (b==pattern) ret++;
  if (c==pattern) ret++;
  
  return ret;
}

Position* TTTState::getAllowedMoves(int &count) {
  count=getAllowedCount();
  int tcount=0;
  if (count==0)
    return NULL;

  Position *pos=new Position[count];
  char *data=m_field->getData();
  
  for (int i=0; i<3; i++) 
    for (int j=0; j<3; j++) 
      if (data[i*3+j]==-1) {
	pos[tcount].setW(i);
	pos[tcount++].setH(j);
      }
  
  return pos;
}

int TTTState::getAllowedCount() {
  if (getWinner()!=-1)
    return 0;
  int maxcount=0;
  char *data=m_field->getData();
  for (int i=0; i<9; i++)
    if (data[i]==-1)
      maxcount++;

  if (maxcount==0)
    setWinner(2);
  
  return maxcount;
}

GameField* TTTState::getField() {
  return m_field;
}

